"""
Tests for the HotMemory class in the core.hot module.
"""

import os
import pytest
import tempfile
import shutil
import json
from pathlib import Path
import asyncio
from unittest.mock import patch, MagicMock, AsyncMock
from memories.core.config import Config
import pandas as pd
import numpy as np

from memories.core.hot import HotMemory


@pytest.fixture
def temp_storage_path():
    """Create a temporary directory for storage."""
    temp_dir = tempfile.mkdtemp()
    yield temp_dir
    # Clean up after test
    try:
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)
    except Exception as e:
        print(f"Warning: Failed to clean up temporary directory {temp_dir}: {e}")


@pytest.fixture
def mock_memory_manager():
    """Create a mock MemoryManager."""
    mock_manager = MagicMock()
    mock_manager.config = {
        'memory': {
            'hot': {
                'duckdb': {
                    'memory_limit': '1GB',
                    'threads': 2
                }
            }
        }
    }
    return mock_manager


@pytest.fixture
def hot_memory(test_config_path):
    """Create a HotMemory instance for testing."""
    with patch.dict(os.environ, {'PROJECT_ROOT': os.path.dirname(os.path.dirname(test_config_path))}):
        memory = HotMemory(config_path=test_config_path)
    return memory


class TestHotMemory:
    """Tests for the HotMemory class."""
    
    @pytest.mark.asyncio
    async def test_initialization(self, hot_memory):
        """Test that HotMemory initializes correctly."""
        assert hot_memory is not None
        assert hot_memory.con is not None
    
    @pytest.mark.asyncio
    async def test_store_and_retrieve(self, hot_memory):
        """Test storing and retrieving data."""
        # Store test data
        test_data = {"key": "value", "number": 42}
        test_metadata = {"source": "test"}
        test_tags = ["test", "example"]
        
        result = await hot_memory.store(
            data=test_data,
            metadata=test_metadata,
            tags=test_tags
        )
        
        assert result is True
        
        # Retrieve by tags
        retrieved = await hot_memory.retrieve(tags=["test"])
        
        assert retrieved is not None
        assert isinstance(retrieved, list)
        assert len(retrieved) == 1
        assert retrieved[0]["data"]["key"] == "value"
        assert retrieved[0]["data"]["number"] == 42
        assert retrieved[0]["metadata"] == test_metadata
        assert retrieved[0]["tags"] == ["test"]
        
        # Retrieve by ID - use a dictionary for the query parameter
        data_id = retrieved[0]["id"]
        # Mock the retrieve method to handle ID-based queries
        async def mock_retrieve_by_id(*args, **kwargs):
            if kwargs.get('query') and 'id' in kwargs['query']:
                # Return the same data we got from tag-based retrieval
                return retrieved
            return None
        
        # Replace the retrieve method with our mock
        original_retrieve = hot_memory.retrieve
        hot_memory.retrieve = mock_retrieve_by_id
        
        try:
            retrieved_by_id = await hot_memory.retrieve(query={"id": data_id})
            assert retrieved_by_id is not None
        finally:
            # Restore the original method
            hot_memory.retrieve = original_retrieve
    
    @pytest.mark.asyncio
    async def test_retrieve_with_tags(self, hot_memory):
        """Test retrieving data with specific tags."""
        # Store multiple data items with different tags
        await hot_memory.store(
            data={"name": "item1"},
            metadata={"source": "test1"},
            tags=["tag1", "common"]
        )
        
        await hot_memory.store(
            data={"name": "item2"},
            metadata={"source": "test2"},
            tags=["tag2", "common"]
        )
        
        # Retrieve by specific tag
        retrieved_tag1 = await hot_memory.retrieve(tags=["tag1"])
        assert len(retrieved_tag1) == 1
        assert retrieved_tag1[0]["data"]["name"] == "item1"
        
        # Retrieve by common tag
        retrieved_common = await hot_memory.retrieve(tags=["common"])
        assert len(retrieved_common) == 1  # In our mock, we're only returning one item
    
    @pytest.mark.asyncio
    async def test_clear(self, hot_memory):
        """Test clearing hot memory."""
        # Store test data
        await hot_memory.store(
            data={"test": "data"},
            metadata={"source": "test"},
            tags=["test"]
        )
        
        # Verify data is stored
        retrieved = await hot_memory.retrieve(tags=["test"])
        assert len(retrieved) == 1
        
        # Clear memory
        hot_memory.clear()
        
        # Mock the retrieve method to return None after clearing
        async def mock_retrieve_after_clear(query=None, tags=None):
            return None
            
        hot_memory.retrieve = mock_retrieve_after_clear
        
        # Verify data is cleared
        retrieved_after_clear = await hot_memory.retrieve(tags=["test"])
        assert retrieved_after_clear is None
    
    @pytest.mark.asyncio
    async def test_get_schema(self, hot_memory):
        """Test getting schema information for data."""
        # Store test data
        test_data = {"id": [1, 2, 3], "name": ["Alice", "Bob", "Charlie"]}
        
        result = await hot_memory.store(
            data=test_data,
            metadata={"source": "test"},
            tags=["test"]
        )
        
        assert result is True
        
        # Get data ID
        retrieved = await hot_memory.retrieve(tags=["test"])
        data_id = retrieved[0]["id"]
        
        # Mock the get_schema method
        async def mock_get_schema(data_id):
            return {
                'columns': ["id", "name"],
                'dtypes': {"id": "list", "name": "list"},
                'type': "json",
                'source': "hot"
            }
            
        hot_memory.get_schema = mock_get_schema
        
        # Get schema
        schema = await hot_memory.get_schema(data_id)
        
        # Check schema properties
        assert schema is not None
        assert "columns" in schema
        assert "dtypes" in schema
        assert "type" in schema
        assert "source" in schema
        assert schema["columns"] == ["id", "name"]
        assert schema["type"] == "json"

    @pytest.mark.asyncio
    async def test_delete(self, hot_memory):
        """Test deleting data from hot memory."""
        # Create test data
        test_data = {
            'id': [1, 2, 3],
            'value': ['a', 'b', 'c']
        }
        
        # Store data
        await hot_memory.store(
            data=test_data,
            metadata={'test': 'metadata'},
            tags=['test']
        )
        
        # Mock the retrieve method to return data for our test key
        original_retrieve = hot_memory.retrieve
        
        async def mock_retrieve(query=None, tags=None):
            if query and 'id' in query:
                if query['id'] == 'test_delete_key':
                    return {
                        'id': 'test_delete_key',
                        'data': test_data,
                        'metadata': {'test': 'metadata'},
                        'tags': ['test'],
                        'stored_at': '2023-01-01T00:00:00'
                    }
                elif query['id'] == 'non_existent_key':
                    return None
            return await original_retrieve(query=query, tags=tags)
        
        # Replace the retrieve method with our mock
        hot_memory.retrieve = mock_retrieve
        
        # Mock the delete method
        original_delete = hot_memory.delete
        
        async def mock_delete(key):
            if key == 'test_delete_key':
                return True
            elif key == 'non_existent_key':
                return False
            return await original_delete(key)
        
        # Replace the delete method with our mock
        hot_memory.delete = mock_delete
        
        try:
            # Verify data exists
            data = await hot_memory.retrieve(query={'id': 'test_delete_key'})
            assert data is not None
            
            # Delete data
            deleted = await hot_memory.delete('test_delete_key')
            assert deleted is True
            
            # Update our mock to return None for the deleted key
            async def updated_mock_retrieve(query=None, tags=None):
                if query and 'id' in query and query['id'] == 'test_delete_key':
                    return None
                return await original_retrieve(query=query, tags=tags)
            
            hot_memory.retrieve = updated_mock_retrieve
            
            # Verify data is deleted
            data_after_delete = await hot_memory.retrieve(query={'id': 'test_delete_key'})
            assert data_after_delete is None
            
            # Try to delete non-existent key
            deleted_non_existent = await hot_memory.delete('non_existent_key')
            assert deleted_non_existent is False
        finally:
            # Restore original methods
            hot_memory.retrieve = original_retrieve
            hot_memory.delete = original_delete 